/***********************************************************************/
/*                                                                     */
/*      ir remote  for DSLR                                            */
/*                                                                     */
/* Copyright by morecat_lab 2012                                       */
/*                                                                     */
/*                         TINY10                                      */
/*                         _______                                     */
/*             NC    PB0 1|       |6 PB3 NC                            */
/*                   GND 2|       |5 Vcc                               */
/*               LED PB1 3|_______|4 PB2 pushbutton (INT0)             */
/*                                                                     */
/*                                                                     */
/***********************************************************************/

#define LED 1
#define PUSHBUTTON 2
#define TESTLED 0

/* CTC MODE TEST */

#include <avr/io.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <avr/interrupt.h>

extern void delay_1u(uint8_t);
extern void delay_100u(uint16_t);
extern void pulse_100u(uint16_t);
extern void pentax_shutter();

#define V_OCR33 116  // 33KHz carrier @ 3V, 119 @ 5V
#define V_OCR38 100  // 38KHz carrier @ 3V, 103 @ 5V
#define V_OCR40 95   // 40KHz caerrer @ 3v, 98 @ 5V

#define ON_OCB0   TCCR0A = 0b00010000
#define OFF_OCB0  TCCR0A = 0b00000000

/* 10 instructions (1.0uS @ 8MHz) delay for 1 loop */
void delay_1u(uint8_t __count) {
  __asm__ volatile (
                "1: dec %0" "\n\t"
		"nop" "\n" "nop" "\n"
		"nop" "\n" "nop" "\n"
		"nop" "\n" "nop" "\n"
                "brne 1b"
                : "=r" (__count)
                : "0" (__count)
		    );
}

/* 100uS delay */
void delay_100u(uint16_t __count) {
  uint16_t i;
  for (i = 0 ; i < __count ; i++) {
    delay_1u(100);
  }
}

void pulse_100u(uint16_t t1) {
  ON_OCB0;
  delay_100u(t1);
  OFF_OCB0;
}

void delay_ms (uint16_t dly)
{
  uint8_t n, d;

  do {
    n = 31;
    do d = PINB; while (--n);
  } while (--dly);
}

EMPTY_INTERRUPT(INT0_vect);

int main() {
  CCP = 0xD8;
  CLKMSR = 0x00;  // internal 8MHz
  CCP = 0xD8;
  CLKPSR = 0x00; // prescale = 1
  CCP = 0xD8;
  OSCCAL = 160;  // ~= 8MHz at 5V

#ifdef TEST  
  DDRB = ((1<<TESTLED) | (1<< LED)); // output
#else
  DDRB = (1<< LED); // output
#endif
  PUEB = (1<< PUSHBUTTON);		/* Pull-up  */
 
  GTCCR = 0;

  /* OC0B (PB1)を初期化 CTC MODE WGSM0 3:0 = 0100 */
  TCCR0A = 0b00010000;
  /* unuse OC0A , OC0B (Toggle OC0B on compare match), CTC  */
  TCCR0B = 0b00001001;
  /* no prescaler (1/1 clock) */

  // CTCの周波数を設定する
  OCR0A = V_OCR38;		/* for 38KHz carrier */
  OFF_OCB0;

  EICRA = (1<<ISC01);		/* falling edge of INT0 */
  EIMSK = (1<<INT0);		/* INT0 enable */
  sei();
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);

  for ( ; ; ) {
    sleep_mode();			/* Enter Power-Down mode */
    if (bit_is_clear(PINB, PUSHBUTTON)) {
      delay_ms(30);
      if (bit_is_set(PINB, PUSHBUTTON)) {
	continue;		/* if chattring, go to sleep again */
      }
#ifdef TEST
      PORTB |= (1<<TESTLED);
#endif
      pentax_shutter();
      delay_ms(50);
      while(bit_is_clear(PINB, PUSHBUTTON)) /* wait until button release */
	;
#ifdef TEST
      PORTB &= ~(1<<TESTLED);
#endif
    }
  }

}

void pentax_shutter() {
  uint8_t i;
  OCR0A = V_OCR38;
  pulse_100u(132);
  delay_100u(32);
  for (i = 0 ; i < 7 ; i++) {
    pulse_100u(10);
    delay_100u(10);
  }
}
